@@ -39,34 +39,3 @@
 
 /* Define if log4cpp library is used. */
 /* #undef XMLTOOLING_LOG4CPP */
-
-/* Define to 1 to disable XML-Security-dependent features. */
-/* #undef XMLTOOLING_NO_XMLSEC */
-
-/* Define if you wish to disable Xalan-dependent features. */
-#define XSEC_NO_XALAN
-
-#include <xercesc/util/XercesVersion.hpp>
-
-#if (XERCES_VERSION_MAJOR >= 3)
-# define XMLTOOLING_XERCESC_COMPLIANT_DOMLS     1
-# define XMLTOOLING_XERCESC_BOOLSETIDATTRIBUTE  1
-# define XMLTOOLING_XERCESC_64BITSAFE           1
-#endif
-
-/* Define to 1 if you have the `xsecsize_t' type. */
-#define HAVE_XSECSIZE_T 1
-
-#ifndef XMLTOOLING_NO_XMLSEC
-# include <xsec/framework/XSECDefs.hpp>
-# if (_XSEC_VERSION_FULL >= 10500)
-#  define XMLTOOLING_XMLSEC_C14N11 1
-# endif
-# if (_XSEC_VERSION_FULL >= 10600)
-#  define XMLTOOLING_XMLSEC_MULTIPLECRL 1
-#  define XMLTOOLING_XMLSEC_SIGALGORITHM 1
-#  define XMLTOOLING_XMLSEC_ECC 1
-#  define XMLTOOLING_XMLSEC_DEBUGLOGGING 1
-# endif
-#endif
-
