@@ -25,7 +25,6 @@
  */
 
 #include "internal.h"
-#include "logging.h"
 #include "io/HTTPResponse.h"
 #include "security/BasicX509Credential.h"
 #include "security/CredentialCriteria.h"
@@ -34,9 +33,10 @@
 #include "security/OpenSSLCredential.h"
 #include "security/SecurityHelper.h"
 #include "security/XSECCryptoX509CRL.h"
+#include "security/impl/ManagedResource.h"
+#include "security/impl/OpenSSLSupport.h"
 #include "util/NDC.h"
 #include "util/PathResolver.h"
-#include "util/Threads.h"
 #include "util/XMLHelper.h"
 
 #include <memory>
@@ -55,92 +55,10 @@ using namespace std;
 using xercesc::DOMElement;
 using xercesc::chLatin_f;
 using xercesc::chDigit_0;
+using boost::scoped_ptr;
 
 namespace xmltooling {
 
-    // The ManagedResource classes handle memory management, loading of the files
-    // and staleness detection. A copy of the active objects is always stored in
-    // these instances.
-
-    class XMLTOOL_DLLLOCAL ManagedResource {
-    protected:
-        ManagedResource() : local(true), reloadChanges(true), filestamp(0), reloadInterval(0) {}
-        ~ManagedResource() {}
-
-        SOAPTransport* getTransport() {
-            SOAPTransport::Address addr("FilesystemCredentialResolver", source.c_str(), source.c_str());
-            string scheme(addr.m_endpoint, strchr(addr.m_endpoint,':') - addr.m_endpoint);
-            SOAPTransport* ret = XMLToolingConfig::getConfig().SOAPTransportManager.newPlugin(scheme.c_str(), addr);
-            if (ret)
-                ret->setCacheTag(&cacheTag);
-            return ret;
-        }
-
-    public:
-        bool stale(Category& log, RWLock* lock=nullptr) {
-            if (local) {
-#ifdef WIN32
-                struct _stat stat_buf;
-                if (_stat(source.c_str(), &stat_buf) != 0)
-                    return false;
-#else
-                struct stat stat_buf;
-                if (stat(source.c_str(), &stat_buf) != 0)
-                    return false;
-#endif
-                if (filestamp >= stat_buf.st_mtime)
-                    return false;
-
-                // If necessary, elevate lock and recheck.
-                if (lock) {
-                    log.debug("timestamp of local resource changed, elevating to a write lock");
-                    lock->unlock();
-                    lock->wrlock();
-                    if (filestamp >= stat_buf.st_mtime) {
-                        // Somebody else handled it, just downgrade.
-                        log.debug("update of local resource handled by another thread, downgrading lock");
-                        lock->unlock();
-                        lock->rdlock();
-                        return false;
-                    }
-                }
-
-                // Update the timestamp regardless. No point in repeatedly trying.
-                filestamp = stat_buf.st_mtime;
-                log.info("change detected, reloading local resource...");
-            }
-            else {
-                time_t now = time(nullptr);
-
-                // Time to reload?
-                if (now - filestamp < reloadInterval)
-                    return false;
-
-                // If necessary, elevate lock and recheck.
-                if (lock) {
-                    log.debug("reload interval for remote resource elapsed, elevating to a write lock");
-                    lock->unlock();
-                    lock->wrlock();
-                    if (now - filestamp < reloadInterval) {
-                        // Somebody else handled it, just downgrade.
-                        log.debug("update of remote resource handled by another thread, downgrading lock");
-                        lock->unlock();
-                        lock->rdlock();
-                        return false;
-                    }
-                }
-
-                filestamp = now;
-                log.info("reloading remote resource...");
-            }
-            return true;
-        }
-
-        bool local,reloadChanges;
-        string format,source,backing,cacheTag;
-        time_t filestamp,reloadInterval;
-    };
-
     class XMLTOOL_DLLLOCAL ManagedKey : public ManagedResource {
     public:
         ManagedKey() : key(nullptr) {}
@@ -153,7 +71,7 @@ namespace xmltooling {
                 nkey = SecurityHelper::loadKeyFromFile(source.c_str(), format.c_str(), password);
             }
             else {
-                auto_ptr<SOAPTransport> t(getTransport());
+                scoped_ptr<SOAPTransport> t(getTransport());
                 log.info("loading private key from URL (%s)", source.c_str());
                 nkey = SecurityHelper::loadKeyFromURL(*t.get(), backing.c_str(), format.c_str(), password);
             }
@@ -163,6 +81,7 @@ namespace xmltooling {
                 format = SecurityHelper::guessEncodingFormat(local ? source.c_str() : backing.c_str());
         }
 
+		string format;
         XSECCryptoKey* key;
     };
 
@@ -178,7 +97,7 @@ namespace xmltooling {
                 SecurityHelper::loadCertificatesFromFile(ncerts, source.c_str(), format.c_str(), password);
             }
             else {
-                auto_ptr<SOAPTransport> t(getTransport());
+                scoped_ptr<SOAPTransport> t(getTransport());
                 log.info("loading certificate(s) from URL (%s)", source.c_str());
                 SecurityHelper::loadCertificatesFromURL(ncerts, *t.get(), backing.c_str(), format.c_str(), password);
             }
@@ -187,6 +106,8 @@ namespace xmltooling {
             if (format.empty())
                 format = SecurityHelper::guessEncodingFormat(local ? source.c_str() : backing.c_str());
         }
+
+		string format;
         vector<XSECCryptoX509*> certs;
     };
 
@@ -202,7 +123,7 @@ namespace xmltooling {
                 SecurityHelper::loadCRLsFromFile(ncrls, source.c_str(), format.c_str());
             }
             else {
-                auto_ptr<SOAPTransport> t(getTransport());
+                scoped_ptr<SOAPTransport> t(getTransport());
                 log.info("loading CRL(s) from URL (%s)", source.c_str());
                 SecurityHelper::loadCRLsFromURL(ncrls, *t.get(), backing.c_str(), format.c_str());
             }
@@ -211,6 +132,8 @@ namespace xmltooling {
             if (format.empty())
                 format = SecurityHelper::guessEncodingFormat(local ? source.c_str() : backing.c_str());
         }
+
+		string format;
         vector<XSECCryptoX509CRL*> crls;
     };
 
@@ -218,7 +141,7 @@ namespace xmltooling {
     class XMLTOOL_DLLLOCAL FilesystemCredentialResolver : public CredentialResolver
     {
     public:
-        FilesystemCredentialResolver(const DOMElement* e);
+        FilesystemCredentialResolver(const DOMElement* e, bool deprecationSupport=true);
         virtual ~FilesystemCredentialResolver();
 
         Lockable* lock();
@@ -235,8 +158,8 @@ namespace xmltooling {
     private:
         Credential* getCredential();
 
-        RWLock* m_lock;
-        Credential* m_credential;
+        scoped_ptr<RWLock> m_lock;
+        auto_ptr<Credential> m_credential;
         string m_keypass,m_certpass;
         unsigned int m_keyinfomask,m_usage;
         bool m_extractNames;
@@ -289,9 +212,9 @@ namespace xmltooling {
     #pragma warning( pop )
 #endif
 
-    CredentialResolver* XMLTOOL_DLLLOCAL FilesystemCredentialResolverFactory(const DOMElement* const & e)
+    CredentialResolver* XMLTOOL_DLLLOCAL FilesystemCredentialResolverFactory(const DOMElement* const & e, bool deprecationSupport)
     {
-        return new FilesystemCredentialResolver(e);
+        return new FilesystemCredentialResolver(e, deprecationSupport);
     }
 
     static const XMLCh backingFilePath[] =  UNICODE_LITERAL_15(b,a,c,k,i,n,g,F,i,l,e,P,a,t,h);
@@ -315,8 +238,8 @@ namespace xmltooling {
     static const XMLCh _use[] =             UNICODE_LITERAL_3(u,s,e);
 };
 
-FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
-    : m_lock(nullptr), m_credential(nullptr), m_keyinfomask(XMLHelper::getAttrInt(e, 0, keyInfoMask)),
+FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e, bool deprecationSupport)
+    : m_keyinfomask(XMLHelper::getAttrInt(e, 0, keyInfoMask)),
         m_usage(Credential::UNSPECIFIED_CREDENTIAL), m_extractNames(true)
 {
 #ifdef _DEBUG
@@ -384,6 +307,7 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
             XMLToolingConfig::getConfig().getPathResolver()->resolve(m_key.source, PathResolver::XMLTOOLING_CFG_FILE);
             m_key.local = true;
             m_key.reloadChanges = XMLHelper::getAttrBool(e, true, _reloadChanges);
+            m_key.m_deprecationSupport = deprecationSupport;
         }
         else if ((e=XMLHelper::getFirstChildElement(keynode,_URL)) && e->hasChildNodes()) {
             prop = e->getFirstChild()->getNodeValue();
@@ -395,6 +319,7 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
                 throw XMLSecurityException("FilesystemCredentialResolver can't access key, backingFilePath missing from URL element.");
             XMLToolingConfig::getConfig().getPathResolver()->resolve(m_key.backing, PathResolver::XMLTOOLING_CACHE_FILE);
             m_key.reloadInterval = XMLHelper::getAttrInt(e, 0, _reloadInterval);
+            m_key.m_deprecationSupport = deprecationSupport;
         }
         else {
             log.error("Path/URL element missing inside Key element");
@@ -420,7 +345,7 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
         while (e) {
             if (e->hasChildNodes()) {
                 m_crls.push_back(ManagedCRL());
-                ManagedResource& crl = m_crls.back();
+                ManagedCRL& crl = m_crls.back();
                 crl.format = crlformat;
                 prop = e->getFirstChild()->getNodeValue();
                 auto_ptr_char crlpath(prop);
@@ -428,6 +353,7 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
                 XMLToolingConfig::getConfig().getPathResolver()->resolve(crl.source, PathResolver::XMLTOOLING_CFG_FILE);
                 crl.local = true;
                 crl.reloadChanges = XMLHelper::getAttrBool(e, true, _reloadChanges);
+                crl.m_deprecationSupport = deprecationSupport;
             }
             e = XMLHelper::getNextSiblingElement(e, Path);
         }
@@ -436,7 +362,7 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
         while (e) {
             if (e->hasChildNodes()) {
                 m_crls.push_back(ManagedCRL());
-                ManagedResource& crl = m_crls.back();
+                ManagedCRL& crl = m_crls.back();
                 crl.format = crlformat;
                 prop = e->getFirstChild()->getNodeValue();
                 auto_ptr_char crlpath(prop);
@@ -447,6 +373,7 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
                     throw XMLSecurityException("FilesystemCredentialResolver can't access CRL, backingFilePath missing from URL element.");
                 XMLToolingConfig::getConfig().getPathResolver()->resolve(crl.backing, PathResolver::XMLTOOLING_CACHE_FILE);
                 crl.reloadInterval = XMLHelper::getAttrInt(e, 0, _reloadInterval);
+                crl.m_deprecationSupport = deprecationSupport;
             }
             e = XMLHelper::getNextSiblingElement(e, _URL);
         }
@@ -467,7 +394,7 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
         while (e) {
             if (e->hasChildNodes() && (XMLString::equals(e->getLocalName(), Path) || XMLString::equals(e->getLocalName(), CAPath))) {
                 m_certs.push_back(ManagedCert());
-                ManagedResource& cert = m_certs.back();
+                ManagedCert& cert = m_certs.back();
                 cert.format = certformat;
                 prop = e->getFirstChild()->getNodeValue();
                 auto_ptr_char certpath(prop);
@@ -475,10 +402,11 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
                 XMLToolingConfig::getConfig().getPathResolver()->resolve(cert.source, PathResolver::XMLTOOLING_CFG_FILE);
                 cert.local = true;
                 cert.reloadChanges = XMLHelper::getAttrBool(e, true, _reloadChanges);
+                cert.m_deprecationSupport = deprecationSupport;
             }
             else if (e->hasChildNodes() && XMLString::equals(e->getLocalName(), _URL)) {
                 m_certs.push_back(ManagedCert());
-                ManagedResource& cert = m_certs.back();
+                ManagedCert& cert = m_certs.back();
                 cert.format = certformat;
                 prop = e->getFirstChild()->getNodeValue();
                 auto_ptr_char certpath(prop);
@@ -489,6 +417,7 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
                     throw XMLSecurityException("FilesystemCredentialResolver can't access certificate, backingFilePath missing from URL element.");
                 XMLToolingConfig::getConfig().getPathResolver()->resolve(cert.backing, PathResolver::XMLTOOLING_CACHE_FILE);
                 cert.reloadInterval = XMLHelper::getAttrInt(e, 0, _reloadInterval);
+                cert.m_deprecationSupport = deprecationSupport;
             }
             e = XMLHelper::getNextSiblingElement(e);
         }
@@ -513,9 +442,8 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
     }
 
     // Load it all into a credential object and then create the lock.
-    auto_ptr<Credential> credential(getCredential());
-    m_lock = RWLock::create();
-    m_credential = credential.release();
+    m_credential.reset(getCredential());
+    m_lock.reset(RWLock::create());
     if (m_credential->getPrivateKey() == nullptr) {
         log.info("no private key resolved, usable for verification/trust only");
     }
@@ -523,15 +451,13 @@ FilesystemCredentialResolver::FilesystemCredentialResolver(const DOMElement* e)
 
 FilesystemCredentialResolver::~FilesystemCredentialResolver()
 {
-    delete m_credential;
-    delete m_lock;
 }
 
 Credential* FilesystemCredentialResolver::getCredential()
 {
     // First, verify that the key and certificate match.
     if (m_key.key && !m_certs.empty()) {
-        auto_ptr<XSECCryptoKey> temp(m_certs.front().certs.front()->clonePublicKey());
+        scoped_ptr<XSECCryptoKey> temp(m_certs.front().certs.front()->clonePublicKey());
         if (!SecurityHelper::matches(*m_key.key, *temp.get()))
             throw XMLSecurityException("FilesystemCredentialResolver given mismatched key/certificate, check for consistency.");
     }
@@ -589,7 +515,7 @@ Lockable* FilesystemCredentialResolver::lock()
 
     bool writelock = false, updated = false;
 
-    if (m_key.stale(log, m_lock)) {
+    if (m_key.stale(log, m_lock.get())) {
         writelock = true;
         try {
             m_key.load(log, m_keypass.c_str());
@@ -610,7 +536,7 @@ Lockable* FilesystemCredentialResolver::lock()
     }
 
     for (vector<ManagedCert>::iterator i = m_certs.begin(); i != m_certs.end(); ++i) {
-        if (i->stale(log, writelock ? nullptr : m_lock)) {
+        if (i->stale(log, writelock ? nullptr : m_lock.get())) {
             writelock = true;
             try {
                 i->load(log, (i==m_certs.begin()) ? m_certpass.c_str() : nullptr);
@@ -632,7 +558,7 @@ Lockable* FilesystemCredentialResolver::lock()
     }
 
     for (vector<ManagedCRL>::iterator j = m_crls.begin(); j != m_crls.end(); ++j) {
-        if (j->stale(log, writelock ? nullptr : m_lock)) {
+        if (j->stale(log, writelock ? nullptr : m_lock.get())) {
             writelock = true;
             try {
                 j->load(log);
@@ -656,8 +582,7 @@ Lockable* FilesystemCredentialResolver::lock()
     if (updated) {
         try {
             auto_ptr<Credential> credential(getCredential());
-            delete m_credential;
-            m_credential = credential.release();
+            m_credential = credential; // swap via auto_ptr
         }
         catch (exception& ex) {
             log.crit("maintaining existing credentials, error reloading: %s", ex.what());
@@ -673,7 +598,7 @@ Lockable* FilesystemCredentialResolver::lock()
 
 const Credential* FilesystemCredentialResolver::resolve(const CredentialCriteria* criteria) const
 {
-    return (criteria ? (criteria->matches(*m_credential) ? m_credential : nullptr) : m_credential);
+    return (criteria ? (criteria->matches(*m_credential) ? m_credential.get() : nullptr) : m_credential.get());
 }
 
 vector<const Credential*>::size_type FilesystemCredentialResolver::resolve(
@@ -681,7 +606,7 @@ vector<const Credential*>::size_type FilesystemCredentialResolver::resolve(
     ) const
 {
     if (!criteria || criteria->matches(*m_credential)) {
-        results.push_back(m_credential);
+        results.push_back(m_credential.get());
         return 1;
     }
     return 0;
